<?php
if(!defined('_PS_ROOT_DIR_')) {
	include_once(dirname(__FILE__).'/smspilot.class.php');
} else {
	include_once(_PS_ROOT_DIR_ . '/modules/smspilot/smspilot.class.php');
}

class SMSPilot extends Module {
	private $_html  = '';
	private $_postErrors;
	private $_balance = null;
	
	/* ************************************ *
	 * STANDARD PRESTASHOP MODULE FUNCTIONS *
	 * ************************************ */
	function __construct() {
		$this->name = 'smspilot';
		
		parent::__construct();
		
		$this->displayName = 'SMSPilot.Ru';
		$this->description = $this->l('Sends notifications using SMSPILOT platform');
		$this->tab = 'Tools';
		$this->version = 1.0;
	}
	
	/**
	 * Installation : register hooks and default configuration values
	 */
	public function install() {
		if (!parent::install() OR
			!$this->registerHook('newOrder'))
//			!$this->registerHook('updateQuantity'))
			return false;
		
		foreach ($this->_getAdminParameters() as $p) {
			if(key_exists('name',$p)) {
				if(!Configuration::updateValue($p['name'],$p['default']))
					return false;
			}
		}
		return true;
	}
	
	/**
	 * Unregister configuration values
	 */
	public function uninstall()
	{
		foreach ($this->_getAdminParameters() as $p) {
			if(key_exists('name',$p)) {
				if(!Configuration::deleteByName($p['name']))
					return false;
			}
		}
		return parent::uninstall();
	}
	
	/* ******************* *
	 * ADMINISTRATION FORM *
	 * ******************* */
	/**
	 * Return administration form
	 */
	public function getContent() {
		if( Tools::isSubmit('smspilotsave') ) {
			
			$this->_postValidation();
			
			if (!sizeof($this->_postErrors)) {
				// Valid form, update values
				foreach ($this->_getAdminParameters() as $p) {
					if($p['type']=='checkbox') {
						$value = Tools::getIsset($p['name']) ? 1 : 0;
						Configuration::updateValue($p['name'], $value);
					}else if(key_exists('name',$p) && $p['type'] != 'static') {
						Configuration::updateValue($p['name'],Tools::getValue($p['name']));
					}
				}
				$this->_html .= '<div class="conf confirm">'.$this->l('Settings updated').'</div>';
				if ($this->_sendSMS($this->l('SMSPilot configuration test')))
					$this->_html .= '<div class="conf confirm">'.$this->l('A test sms has been sent to your phone').'</div>';
				else
					$this->_html .= '<div class="alert error">'.$this->l('An error occured when sending a test sms').'</div>';
			}
			else {
				// Display form error
				foreach ($this->_postErrors as $err) {
					$this->_html .= '<div class="alert error">'.$err.'</div>';
				}
			}
		}
		
		$this->_displayForm();
		
		return $this->_html;
	}
	
	/**
	 * Check inputs
	 * @return array the list of errors found
	 */
	private function _postValidation() {
		$this->_postErrors = array();
		
		foreach($this->_getAdminParameters() as $p) {
			if(array_key_exists('validate',$p)) {
				$validate_function = $p['validate'];
				$error = $this->$validate_function(Tools::getValue($p['name']));
				if($error != null)
					$this->_postErrors[] = $error;
			}
		}
	}
	private function _checkAPIKEY($input) {
		if (!preg_match('~^[A-Z0-9]{64}$~', $input))
			return $this->l('Invalid APIKEY');
		return NULL;
	}
	private function _checkPhone($input) {

		if(!Validate::isPhoneNumber($input))
			$error = $this->l('Invalid phone number');
			
		return NULL;
	}
	
/*	
	private function _checkUser($input) {
		if(!preg_match('#^[A-Za-z0-9]{0,30}$#',$input) && !Validate::isEmail($input))
			return $this->l('Invalid user name');
		return null;
	}
	
	private function _checkPassword($input) {
		if(!preg_match('#^[A-Za-z0-9]{0,30}$#',$input))
			return $this->l('Invalid password');
		return null;
	}
	
	private function _checkPhone($input) {
		$error = null;
		$numbers = explode(';',$input);
		foreach($numbers as $num)
		{
			if(!Validate::isPhoneNumber($num))
				$error = $this->l('Invalid phone number(s)');
		}
		return $error;
	}
	
	private function _checkOrderText($input) {
		$tokens = array('{price}','{currency}','{payment}','{shopname}','{orderid}','{firstname}','{lastname}');
		$input = str_replace($tokens,'',$input);
		if(!LYRASMS_API::isValidText($input))
			return $this->l('Invalid "new order" text');
		return null;
	}
	
	private function _checkStockText($input) {
		$tokens = array('{shopname}','{productname}','{variant}','{productid}','{variantid}','{stock}','{stocklimit}');
		$input = str_replace($tokens,'',$input);
		if(!LYRASMS_API::isValidText($input))
			return $this->l('Invalid "stock alert" text');
		return null;
	}
*/
	/**
	 * Build administration form's html, using $this->_parameters
	 */
	private function _displayForm() {
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">';
		foreach ($this->_getAdminParameters() as $p) {
			$value = key_exists('name',$p) ? Configuration::get($p['name']) : $p['default'];
			$type = $p['type'];
			switch ($type) {
				case 'text':
				case 'password':
				case 'checkbox':
					$extra  = ($type == 'checkbox' && $value == 1) ? ' checked="checked"' : '';
					$extra .= ($type != 'checkbox') ? ' value="'.$value.'"' : '';
					
					if ($p['name'] == 'SMSPILOT_APIKEY')
						$extra .= ' size="80"';
					
					$this->_html .= '<label>'.$p['label'].'</label>';
					$this->_html .= '<div class="margin-form"><input type="'.$type.'" name="'.$p['name'].'"'.$extra.'/>';
					$this->_html .= isset($p['description']) ? '<p>'.$p['description'].'</p></div>' : '</div>';
				break;
				
				case 'textarea':
					$this->_html .= '<label>'.$p['label'].'</label><div class="margin-form"><textarea rows="2" cols="65" name="'.$p['name'].'" value="'.$value.'">'.$value.'</textarea>';
					$this->_html .= isset($p['description']) ? '<p>'.$p['description'].'</p></div>' : '</div>';
				break;
				
				case 'static':
					$this->_html .= '<label>'.$p['label'].'</label>';
					$this->_html .= '<div class="margin-form" style="color:#000000; font-size:12px; margin-top:3px;">'.$value;
					$this->_html .= isset($p['description']) ? '<p>'.$p['description'].'</p></div>' : '</div>';
				break;
				
				case 'begin_fieldset':
					$this->_html .= '<fieldset><legend>'.$p['label'].'</legend>';
				break;
				
				case 'end_fieldset':
					$this->_html .= '</fieldset><br/>';
				break;
				
				case 'submit':
				default:
					$name = $p['submitname'] ? $p['submitname'] : 'lyrasmsupdate';
					$value = $p['label'] ? $p['label'] : $this->l('Update settings');
					$this->_html .= '<div class="margin-form"><input class="button" name="'.$name.'" value="'.$value.'" type="submit" /></div>';
				break;
			}
			$this->_html .= '<div style="clear:both; font-size:12px;"></div>';
		}
		$this->_html .= '</form>';
	}
	
	/* ************** *
	 * HOOK FUNCTIONS *
	 * ************** */
	public function hookNewOrder($params) {
//		error_reporting(E_ALL);
//		file_put_contents( dirname(__FILE__).'/data.txt', print_r($params, true));

		$order = $params['order'];
		$customer = $params['customer'];
		$currency = $params['currency'];
			
		$this->_sendSMS( 'New order: '.$order->total_paid.$currency->iso_code.' '.$order->payment.' #'.$order->id.' '.$customer->firstname.' '.$customer->lastname );

	}
	/*
	
	public function hookUpdateQuantity($params) {
		$qty = intval($params['product']['quantity_attribute'] ? $params['product']['quantity_attribute'] : $params['product']['stock_quantity']) - intval($params['product']['quantity']);
		$limit = Configuration::get('LYRASMS_LOW_STOCK_LIMIT');
		
		if(Configuration::get('LYRASMS_LOW_STOCK') && $qty<=$limit) {
			$this->_sendSMS(
				array(	'{shopname}'=>Configuration::get('PS_SHOP_NAME'),
						'{productname}'=>strval($params['product']['name']),
						'{variant}'=>strval($params['product']['attributes_small']),
						'{productid}'=>$params['product']['id_product'],
						'{variantid}'=>$params['product']['id_product_attribute'],
						'{stock}'=>$qty,
						'{stocklimit}'=>$limit
				),
				$phone,
				'neworder'.$order->id
			);
		}
	}
	*/
	
	/* ***************** *
	 * UTILITY FUNCTIONS *
	 * ***************** */
	/**
	 * Returns a list of arrays representing the administration form
	 */
	private function _getAdminParameters() {
		/*
		$order_text = $this->l('{firstname} {lastname} has made an order of {price}{currency} on {shopname} using {payment}. Order id : {orderid}');
		$order_text_description = $this->l('Available parameters : ').
				$this->l('price').' {price}, '.$this->l('currency').' {currency}, '.$this->l('payment').' {payment}, '.
				$this->l('shop name').' {shopname}, '.$this->l('order id').' {orderid}, '.
				$this->l('client first name').' {firstname}, '.$this->l('client last name').' {lastname}'
		;
		
		$stock_text = $this->l('{shopname} : stock alert for product {productname} {variant} (id:{productid}-{variantid}) : {stock} left');
		$stock_text_description = $this->l('Available parameters : ').
				$this->l('shop name').' {shopname}, '.$this->l('product name').' {productname}, '.$this->l('variant').'{variant}, '.
				$this->l('product id').' {productid}, '.$this->l('variant id').'{variantid}, '.
				$this->l('stock left').' {stock}, '.$this->l('stock alert limit').' {stocklimit}'
		;
*/		
//		$apikey = Configuration::get('SMSPILOT_APIKEY');
//		$phone = Configuration::get('SMSPILOT_PHONE');
//		$password = Configuration::get('LYRASMS_PASSWORD');
		/*
		if($user && $password) {
			$lyrasms_link_description = null;
			$lyrasms_url = 'http://www2.lyra-sms.com/bulksms/authentificationAbonneAction.do?email='.$user.'&password='.$password.'&forward=accueil&action=add';
		}
		else {
			$lyrasms_link_description = $this->l('Subscribe using this url and get an exclusive offer !');
			$lyrasms_url = 'http://ecommerce.lyra-sms.com/';
		}
		$lyrasms_link = '<a href="'.$lyrasms_url.'" style="text-decoration:underline;">ecommerce.lyra-sms.com</a>';
		
		$user_description = $this->l('You need to be registered on ').$lyrasms_link.$this->l(' to use this module.');
		*/
		
		$parameters = array(
			array('type'=>'begin_fieldset',	'label'=>$this->l('General parameters')),
				array('type'=>'text', 'label'=>$this->l('SMSPILOT APIKEY'), 					'name'=>'SMSPILOT_APIKEY',			'default'=>'', 'validate'=>'_checkAPIKEY',
						'description'=> '<a href="http://www.smspilot.ru/my.php" target="_blank" title="Open in new window">Get APIKEY</a>'),
				array('type'=>'text', 'label'=>$this->l('Phone number'),					'name'=>'SMSPILOT_PHONE',		'default'=>'', 'validate'=>'_checkPhone',
						'description'=>$this->l('ex. 79087964781')),
				array('type'=>'submit', 'submitname'=>'smspilotsave', 'label'=>$this->l('Save and test this configuration')),
				array('type'=>'static', 'label'=>$this->l('SMS Balance'),			'default'=>$this->_getBalance()),
			array('type'=>'end_fieldset'),

			array('type'=>'begin_fieldset',	'label'=>$this->l('About SMSPilot.Ru')),
				array('type'=>'static', 	'label'=>$this->l('Developed by'), 					'default'=>'Sergey Schuchkin (smspilot)'),
				array('type'=>'static', 	'label'=>$this->l('Website to buy SMS packages'),	'default'=> '<a href="http://www.smspilot.ru/my.php" target="_blank" title="Open in new window">http://www.smspilot.ru/my.php</a>'),
			array('type'=>'end_fieldset')
		);
		return $parameters;
	}
	
	private function _sendSMS( $text ) {
		if (Configuration::get('SMSPILOT_APIKEY') && Configuration::get('SMSPILOT_PHONE')) {
			$sms = new _SMSPilot( Configuration::get('SMSPILOT_APIKEY') );
			return $sms->send( Configuration::get('SMSPILOT_PHONE'), $text );
		} else
			return false;
	}
	
	private function _getBalance() {
		if($this->_balance !== NULL && $this->_balance !== "-") {
			return $this->_balance;
		}
		$apikey = Configuration::get('SMSPILOT_APIKEY');
		if ($apikey) {
			$sms = new _SMSPilot( $apikey );
			if ($b = $sms->balance())
				$this->_balance = $b;
			else
				$this->_balance = $sms->error;
		} else
			$this->_balance = '-';

		return $this->_balance;
	}
}
?>